import tensorflow as tf
from tensorflow.keras.models import Model
from tensorflow.keras.layers import Input, Conv2D, MaxPooling2D, Dense, Dropout, Flatten
from tensorflow.keras.layers import concatenate
from tensorflow.keras.layers import BatchNormalization
from tensorflow.keras.callbacks import TensorBoard, EarlyStopping
import matplotlib.pyplot as plt

#from keras import backend as kb
#from sklearn.preprocessing import MinMaxScaler
#from sklearn.model_selection import train_test_split
#from keras.regularizers import l2


# normalize?

def basic_NN_test(x_data, y_data, seed_number):
    tf.random.set_seed(seed_number)

    x1_input = Input(shape=(x_data.shape[1],))
    x = Flatten()(x1_input)
    x = Dense(32, activation='relu')(x)
    x = Dense(64, activation='relu')(x)
    x = Dense(64, activation='relu')(x)
    x = Dense(64, activation='relu')(x)
    x = Dense(64, activation='relu')(x)
    x = Dense(32, activation='relu')(x)

    output = Dense(1, activation='linear')(x)

    model = Model(inputs=[x1_input], outputs=output)

    print(model.summary())
    opt = tf.keras.optimizers.Adam(learning_rate=0.00005)

    model.compile(loss='mean_squared_error', optimizer=opt)

    es = EarlyStopping(monitor='loss', patience=40)

    tb = TensorBoard(log_dir='./logs', histogram_freq=10,
                     write_graph=True, write_images=True, write_grads=True)

    history_callback = model.fit(x=[x_data], y=y_data, epochs=300, validation_split=0.2, shuffle=True, callbacks=[tb, es])

    loss = history_callback.history["loss"]
    val_loss = history_callback.history["val_loss"]

    plt.figure()
    epochs = range(1, len(loss) + 1)
    plt.semilogy(epochs, loss, 'b', label='Training loss')
    plt.semilogy(epochs, val_loss, 'r', label='Validation loss')
    plt.title('Training and validation loss for NN')
    plt.legend()
    filename = f'./nn_model_{1}/loss_{seed_number}.png'
    plt.savefig(filename)
    plt.close()

    model.save(f'./nn_model_{1}/nn_model_{seed_number}.h5')

    return(history_callback)


