import numpy as np
import matplotlib.pyplot as plt
from scipy.integrate import RK45, odeint

def dU_dt(U, t):
    return (U[1],-1*U[1]-0.5*U[0]-1*U[0]*U[0]*U[0]-0.1*U[0]*U[0]*U[0]*U[0]*U[0]+3*np.cos(0.4*t))

def get_hx(t, lam, omega):
    return (lam*np.cos(omega*t))

def get_gx(x):
    return (x*x*x)

def generator(x0, x0_dot, t_end, n_steps):
    lam = 3
    omega = 0.4
    a = 0.5
    b = 1
    d = 1
    U0 = [x0, x0_dot]
    ts=np.linspace(0,t_end, n_steps)
    Us = odeint(dU_dt, U0, ts)
    allx=Us[:,0]
    allx_dot = Us[:,1]
    allx_ddot = [0] * len(allx)
    all_hx = [0]*len(ts)
    all_gx = [0] * len(allx)
    for i in range(len(ts)):
        all_hx[i] = get_hx(ts[i], lam, omega)
    for i in range(len(allx)):
        all_gx[i] = get_gx(allx[i])
    for i in range(len(allx)):
            _, allx_ddot[i] = dU_dt((Us[i, 0], Us[i, 1]), ts[i])
    ts = np.array(ts)
    """
    plt.figure()
    plt.plot(allx)
    plt.show ()
    """

    return allx, allx_dot, allx_ddot, ts, all_hx, all_gx
